%Copyright Colin O'Flynn
%Recreates figure 1 from
%http://ptgmedia.pearsoncmg.com/images/art_sklar7_reed-solomon/elementLinks/art_sklar7_reed-solomon.pdf
%
%"Pb versus p for 32-ary Orthogonal signaling and n = 31, t-error-correcting
% Reed-Solomon Coding. (Reprinted from Data Communications, Networks and
% Systems, ed. Thomas C. Bartee, Howard W. Sams Company, Indianapolis,
% Ind., 1985, p. 311. Originally published in J. P. Odenwalder, Error
% Control Coding Handbook, M/A-COM LINKABIT, Inc., San Diego, Calif.,
% July 15,1976, p. 91.)"
%
% Same as figure 8.1 in Sklar's Digital Communications book

m = 5;
trange = [1 2 4 8];
spec = {'-', '--', ':', '-.'};
prange = 10.^(-4:0.5:-1);
pb = zeros(size(prange));
q=1;
for t=trange
    i = 1;
    for p=prange
        sum = 0;
        for j=(t+1):(2^m - 1)
            sum = sum + j* nchoosek(2^m-1, j) * p^j * (1-p)^(2^m - 1 - j);
        end
        pe = 1/(2^m - 1) * sum;

        pb(i) = pe * (2^(m - 1)) / (2^m - 1);    
        i = i + 1;
    end
    loglog(prange, pb, spec{q});
    hold on
    q=q+1;
end

axis([1E-4 1E-1 1E-7 1E-2])
set(gca,'xdir','reverse')
xlabel('Channel symbol error probability, p');
ylabel('Bit error probability, P_B');
title('Figure 1 from Handout');
legend('t=1', 't=2', 't=4', 't=8');