%Copyright Colin O'Flynn
%Recreates figure 4 from
%http://ptgmedia.pearsoncmg.com/images/art_sklar7_reed-solomon/elementLinks/art_sklar7_reed-solomon.pdf
%
%"Reed-Solomon, rate 7/8, decoder performance as a function of symbol
%size."
%
% Same as figure 8.4 in Sklar's Digital Communications book

mrange = [5 6 7 8];
trange = [2 4 8 16];
spec = {'-', '--', ':', '-.'};
prange = 10.^(-6:0.5:-1);
pb = zeros(size(prange));
q=1;
for t=trange
    m = mrange(q);
    i = 1;
    for p=prange       
        
        %Convert from bit to symbol error       
        pc = 1-(1-p)^m;
        
        sum = 0;        
        for j=(t+1):(2^m - 1)
            sum = sum + j* nchoosek(2^m-1, j) * pc^j * (1-pc)^(2^m - 1 - j);
        end
        pe = 1/(2^m - 1) * sum;

        pb(i) = pe * (2^(m - 1)) / (2^m - 1);    
        i = i + 1;
    end
    loglog(prange, pb, spec{q});
    hold on
    q=q+1;
end
axis([1E-6 1E-1 1E-20 1E-1])
xlabel('Random channel-bit-error probability');
ylabel('Output-bit-error probability');
title('Figure 4 from Handout');
legend('R-S(32,28),m=5', 'R-S(64,56),m=6', 'R-S(128,112),m=7', 'R-S(256,224),m=8');